@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 """
 This module defines the core of the framework, including the
 abstraction of ``Neurons``, ``Synapses``, ``Monitor``, ``Network``,
@@ -12,8 +14,7 @@ synapses, etc.
 from npbrain.core.monitor import *
 from npbrain.core.network import *
 from npbrain.core.neuron import *
-from npbrain.core.ode import *
-from npbrain.core.sde import *
 from npbrain.core.synapse import *
+from npbrain.core.integrator import *
 
 
