@@ -1,10 +1,13 @@
+# -*- coding: utf-8 -*-
+
+import numba as nb
 import numpy as np
 
-from npbrain import profile
-from npbrain.utils import helper
+from npbrain.utils import helper, profile
 
 __all__ = [
     'judge_spike',
+    'get_spike_judger',
     'initial_neu_state',
     'format_geometry',
     'format_refractory',
@@ -12,18 +15,16 @@ __all__ = [
     'generate_fake_neuron'
 ]
 
-neuron_no = 0
 
-
-@helper.autojit
-def judge_spike(neu_state, Vth, t):
+@nb.jit(**profile.get_numba_profile())
+def judge_spike(neu_state, vth, t):
     """Judge and record the spikes of the given neuron group.
 
     Parameters
     ----------
     neu_state : np.ndarray
         The state of the neuron group.
-    Vth : float, int, np.ndarray
+    vth : float, int, np.ndarray
         The spike threshold.
     t : float
         The current time point.
@@ -33,7 +34,7 @@ def judge_spike(neu_state, Vth, t):
     spike_indexes : list
         The neuron indexes that are spiking.
     """
-    above_threshold = (neu_state[0] >= Vth).astype(np.float64)
+    above_threshold = (neu_state[0] >= vth).astype(np.float64)
     prev_above_th = neu_state[-4]
     spike_st = above_threshold * (1. - prev_above_th)
     spike_idx = np.where(spike_st > 0.)[0]
@@ -43,6 +44,21 @@ def judge_spike(neu_state, Vth, t):
     return spike_idx
 
 
+def get_spike_judger():
+    @helper.autojit
+    def f(neu_state, vth, t):
+        above_threshold = (neu_state[0] >= vth).astype(np.float64)
+        prev_above_th = neu_state[-4]
+        spike_st = above_threshold * (1. - prev_above_th)
+        spike_idx = np.where(spike_st > 0.)[0]
+        neu_state[-4] = above_threshold
+        neu_state[-3] = spike_st
+        neu_state[-2][spike_idx] = t
+        return spike_idx
+
+    return f
+
+
 def initial_neu_state(num_var, num_neuron):
     """Initialize the state of the given neuron group.
 
@@ -86,17 +102,21 @@ def format_geometry(geometry):
     Returns
     -------
     num_and_geo : tuple
-        (Number of neurons, geometry of the neuron group).
+        (Number of neurons, geometry of the neuron group),
+        where the shape of geometry is (height, width).
     """
     # define network geometry
     if isinstance(geometry, (int, float)):
-        geometry = (1, int(geometry), 1)
+        geometry = (1, int(geometry))
     elif isinstance(geometry, (tuple, list)):
         # a tuple is given, can be 1 .. N dimensional
-        width = int(geometry[0])
-        height = int(geometry[1]) if len(geometry) >= 2 else 1
-        depth = int(geometry[2]) if len(geometry) >= 3 else 1
-        geometry = (width, height, depth)
+        if len(geometry) == 1:
+            height, width = 1, geometry[0]
+        elif len(geometry) == 2:
+            height, width = geometry[0], geometry[1]
+        else:
+            raise ValueError('Do not support 3+ dimensional networks.')
+        geometry = (height, width)
     else:
         raise ValueError()
     num = int(np.prod(geometry))
@@ -133,16 +153,31 @@ def format_refractory(ref=None):
     return tau_ref
 
 
+class NeuronsView(object):
+    def __init__(self, target, ranks):
+        self.target = target
+        self.ranks = ranks
+        self.num = len(ranks)
+
+
+_neuron_no = 0
+
+
 class Neurons(object):
     """The base neurons class.
 
     Parameters
     ----------
     kwargs : dict
-        Parameters of the neuron group.
+        Parameters of the given neuron group.
     """
 
+    default_variables = [('Isyn', -1), ('spike_time', -2), ('spike', -3),
+                         ('above_threshold', -4), ('not_refractory', -5)]
+
     def __init__(self, **kwargs):
+        if 'args' in kwargs:
+            kwargs.pop('args')
         if 'kwargs' in kwargs:
             kwargs.pop('kwargs')
         for k, v in kwargs.items():
@@ -151,45 +186,45 @@ class Neurons(object):
         # define external connections
         self.pre_synapses = []
         self.post_synapses = []
-        self.pre_groups = []
-        self.post_groups = []
 
         # check functions
         assert 'update_state' in kwargs
         self.update_state = helper.autojit(self.update_state)
 
-        # check `dt`
-        if 'dt' not in kwargs:
-            self.dt = profile.get_dt()
+        # check `geometry`
+        assert 'geometry' in kwargs, 'Must define "geometry".'
+        assert 'num' in kwargs, 'Must define "num".'
 
         # check `name`
         if 'name' not in kwargs:
-            global neuron_no
-            self.name = "Neurons-{}".format(neuron_no)
-            neuron_no += 1
+            global _neuron_no
+            self.name = "Neurons-{}".format(_neuron_no)
+            _neuron_no += 1
+
+        # check `state`
+        assert 'state' in kwargs, 'Must define "state".'
 
         # check `var2index`
         if 'var2index' not in kwargs:
             raise ValueError('Must define "var2index".')
         assert isinstance(self.var2index, dict), '"var2index" must be a dict.'
-        default_variables = [('V', 0), ('Isyn', -1), ('spike_time', -2), ('spike', -3),
-                             ('above_threshold', -4), ('not_refractory', -5)]
-        for k, _ in default_variables:
+        for k, _ in self.default_variables:
             if k in self.var2index:
                 if k == 'V':
-                    assert self.var2index['V'] == 0, 'The position of "V" must be 0.'
+                    if self.var2index['V'] != 0:
+                        print('The position of "V" is not 0.')
                 else:
                     raise ValueError('"{}" is a pre-defined variable, cannot '
                                      'be defined in "var2index".'.format(k))
         user_defined_variables = sorted(list(self.var2index.items()), key=lambda a: a[1])
-        neu_variables = user_defined_variables + default_variables
-        var_index = np.zeros((len(neu_variables),), dtype=np.int32)
+        neu_variables = user_defined_variables + self.default_variables
+        var2index_array = np.zeros((len(neu_variables),), dtype=np.int32)
         vars = dict()
         for i, (var, index) in enumerate(neu_variables):
-            var_index[i] = index
+            var2index_array[i] = index
             vars[var] = i
-        self.vars = vars
-        self.var_index = var_index
+        self.var2index = vars
+        self.var2index_array = var2index_array
 
     def __str__(self):
         return self.name
@@ -197,9 +232,37 @@ class Neurons(object):
     def __repr__(self):
         return self.name
 
+    def __getitem__(self, item):
+        if isinstance(item, int):  # a single neuron
+            return NeuronsView(self, [int(item)])
+        elif isinstance(item, (list, np.ndarray)):
+            if isinstance(item, np.ndarray):
+                if item.ndim != 1:
+                    raise ValueError('Only one-dimensional lists/arrays are allowed to address a neuron group.')
+                item = list(item.astype(int))
+            return NeuronsView(self, list(item))
+        elif isinstance(item, slice):
+            start, stop, step = item.start, item.stop, item.step
+            if item.start is None:
+                start = 0
+            if item.stop is None:
+                stop = self.num
+            if item.step is None:
+                step = 1
+            rk_range = list(range(start, stop, step))
+            return NeuronsView(self, rk_range)
+        else:
+            raise ValueError('Can not address the population with', item)
+
     @property
     def available_monitors(self):
-        return sorted(list(self.variables.keys()))
+        return sorted(list(self.var2index.keys()))
+
+    def set_state(self, key, value):
+        if key not in self.var2index:
+            raise ValueError('Variable "{}" is not in the neuron group.'.format(key))
+        idx = self.var2index[key]
+        self.state[idx] = value
 
 
 def generate_fake_neuron(num, V=0.):
@@ -217,12 +280,10 @@ def generate_fake_neuron(num, V=0.):
     neurons : dict
         An instance of ``Dict`` for simulating neurons.
     """
-    neu = helper.Dict(state=np.zeros((5, num)), num=num,
-                      post_groups=[], pre_groups=[],
-                      pre_synapses=[], post_synapses=[])
-    neu.state[0] = V
-    neu.update_state = helper.autojit(lambda neu_state, t: 1)
-    global neuron_no
-    neu.name = 'FakeNeurons-{}'.format(neuron_no)
-    neuron_no += 1
-    return neu
+
+    var2index = dict(V=0)
+    num, geometry = num, (num, )
+    state = np.zeros((5, num))
+    state[0] = V
+    update_state = helper.autojit(lambda neu_state, t: 1)
+    return Neurons(**locals())
