@@ -1,9 +1,8 @@
+# -*- coding: utf-8 -*-
+
 import numpy as np
 
-from npbrain import profile
 from npbrain.core.synapse import *
-from npbrain.utils.helper import check_params
-from npbrain.utils import conn
 
 __all__ = [
     'GapJunction',
@@ -11,7 +10,7 @@ __all__ = [
 ]
 
 
-def GapJunction(pre, post, weights, connection, **kwargs):
+def GapJunction(pre, post, weights, connection, delay=None, name='gap_junction'):
     """Gap junction, or, electrical synapse.
 
     Parameters
@@ -24,67 +23,48 @@ def GapJunction(pre, post, weights, connection, **kwargs):
         The weighted coefficients of synapses.
     connection : dict, str, callable
         The connection method.
-    kwargs : dict
-        Synapse parameters.
+    delay : None, float
+        The delay length.
+    name : str
+        The name of synapse.
 
     Returns
     -------
     synapse : Synapses
         The constructed electrical synapses.
     """
-    # base
-    # ----
+
     num_pre = pre.num
     num_post = post.num
-    dt = profile.get_dt()
-
-    # essential
-    # ---------
-    name = kwargs.pop('name', 'gap_junction')
     var2index = {'pre_V': (0, 0), 'post_V': (1, -1)}
-    delay = kwargs.pop('delay', None)
-    delay_len = format_delay(delay, dt)
 
-    # connections
-    # -----------
-    pre_indexes, post_indexes, pre_anchors = conn.format_connection(
-        connection, num_pre=num_pre, num_post=num_post)
-    num = len(pre_indexes)
+    pre_ids, post_ids, anchors = connection
+    num = len(pre_ids)
+
     # The first last (num_pre, ) shape variable is "pre-neuron spike"
     # The second last (num_pre, ) shape variable is "pre-neuron potential"
     # The first last (num_post, ) shape variable is "post-neuron potential"
-    state = initial_syn_state(delay_len, num_pre, num_post, num,
-                              num_pre_shape_var=1,
-                              num_post_shape_var=1)
-    check_params(kwargs)
+    state = initial_syn_state(delay, num_pre, num_post, num,
+                              num_pre_shape_var=1, num_post_shape_var=1)
 
     # weights
-    # -------
-    if isinstance(weights, (int, float)):
+    if np.size(weights) == 1:
         weights = np.ones(num) * weights
-    elif isinstance(weights, np.ndarray):
-        if np.size(weights) == 1:
-            weights = np.ones(num) * weights
-        elif np.size(weights) == num:
-            weights = weights
-        else:
-            raise ValueError('Unknown weights shape.')
+    elif np.size(weights) == num:
+        weights = weights
     else:
-        raise ValueError('Unknown weights type.')
+        raise ValueError('Unknown weights shape.')
 
-    # functions
-    # ---------
-    def update_state(syn_state, var_index, t):
+    def update_state(syn_state, t, var_index):
         # get synapse state
         pre_v = syn_state[0][-2]
         post_v = syn_state[1][-1]
         # get gap junction value
         g = np.zeros(num_post)
         for i_ in range(num_pre):
-            start, end = pre_anchors[:, i_]
-            post_idx = post_indexes[start: end]
-            g[post_idx] += weights[start: end] * (pre_v[i_] - post_v[post_idx])
-        # record
+            idx = anchors[:, i_]
+            post_idx = post_ids[idx[0]: idx[1]]
+            g[post_idx] += weights[idx[0]: idx[1]] * (pre_v[i_] - post_v[post_idx])
         record_conductance(syn_state, var_index, g)
 
     def output_synapse(syn_state, var_index, post_neu_state):
@@ -104,7 +84,7 @@ def GapJunction(pre, post, weights, connection, **kwargs):
     return Synapses(**locals())
 
 
-def GapJunction_LIF(pre, post, weights, connection, **kwargs):
+def GapJunction_LIF(pre, post, weights, connection, k_spikelet=0.1, delay=None, name='gap_junction'):
     """Gap junction, or, electrical synapse for LIF neuron model.
 
     Parameters
@@ -117,63 +97,42 @@ def GapJunction_LIF(pre, post, weights, connection, **kwargs):
         The weighted coefficients of synapses.
     connection : dict, str, callable
         The connection method.
-    kwargs : dict
-        Synapse parameters.
+    k_spikelet : float
+        The spikelet factor.
+    delay : None, float
+        The delay length.
+    name : str
+        The name of synapse.
 
     Returns
     -------
     synapse : Synapses
         The constructed electrical synapses.
     """
-    # base
-    # ----
     num_pre = pre.num
     num_post = post.num
-    dt = profile.get_dt()
-
-    # essential
-    # ---------
-    name = kwargs.pop('name', 'gap_junction_for_LIF')
     var2index = {'pre_V': (0, 0), 'post_V': (1, -1)}
-    delay = kwargs.pop('delay', None)
-    delay_len = format_delay(delay, dt)
-
-    # parameters
-    # -----------
-    k_spikelet = kwargs.pop('k_spikelet', 0.1)
     k = k_spikelet * weights
-    check_params(kwargs)
 
-    # connections
-    # -----------
-    pre_indexes, post_indexes, pre_anchors = conn.format_connection(
-        connection, num_pre=num_pre, num_post=num_post)
-    num = len(pre_indexes)
+    pre_ids, post_ids, anchors = connection
+    num = len(pre_ids)
+
     # The first last (num_pre, ) shape variable is "pre-neuron spike"
     # The second last (num_pre, ) shape variable is "pre-neuron potential"
     # The first last (num_post * 2, ) shape variable is "post-neuron potential"
     # Other (num_post * 2, ) shape variables are corresponding for delays
-    state = initial_syn_state(delay_len, num_pre, num_post * 2, num,
-                              num_pre_shape_var=1,
-                              num_post_shape_var=1)
+    state = initial_syn_state(delay, num_pre, num_post * 2, num,
+                              num_pre_shape_var=1, num_post_shape_var=1)
 
     # weights
-    # -------
-    if isinstance(weights, (int, float)):
+    if np.size(weights) == 1:
         weights = np.ones(num) * weights
-    elif isinstance(weights, np.ndarray):
-        if np.size(weights) == 1:
-            weights = np.ones(num) * weights
-        elif np.size(weights) == num:
-            weights = weights
-        else:
-            raise ValueError('Unknown weights shape.')
+    elif np.size(weights) == num:
+        weights = weights
     else:
-        raise ValueError('Unknown weights type.')
+        raise ValueError('Unknown weights shape.')
 
-    # functions
-    # ---------
-    def update_state(syn_state, var_index, t):
+    def update_state(syn_state, t, var_index):
         # get synapse state
         spike = syn_state[0][-1]
         pre_v = syn_state[0][-2]
@@ -184,18 +143,17 @@ def GapJunction_LIF(pre, post, weights, connection, **kwargs):
         # get spikelet
         g1 = np.zeros(num_post)
         for i_ in spike_idx:
-            start, end = pre_anchors[:, i_]
-            post_idx = post_indexes[start: end]
+            idx = anchors[:, i_]
+            post_idx = post_ids[idx[0]: idx[1]]
             g1[post_idx] += k
         g[num_post:] = g1
         # get gap junction value
         g2 = np.zeros(num_post)
         for i_ in range(num_pre):
-            start, end = pre_anchors[:, i_]
-            post_idx = post_indexes[start: end]
-            g2[post_idx] += weights[start: end] * (pre_v[i_] - post_v[post_idx])
+            idx = anchors[:, i_]
+            post_idx = post_ids[idx[0]: idx[1]]
+            g2[post_idx] += weights[idx[0]: idx[1]] * (pre_v[i_] - post_v[post_idx])
         g[:num_post] = g2
-        # record
         record_conductance(syn_state, var_index, g)
 
     def output_synapse(syn_state, var_index, post_neu_state):
