@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 
+# profile.py
+from npbrain.utils.profile import *
 
-# conn.py
-from npbrain.utils.conn import *
+# connect.py
+from npbrain.utils.connect import *
 
 # input_factory.py
 from npbrain.utils.input_factory import *
@@ -13,14 +15,11 @@ from npbrain.utils.helper import *
 # measure.py
 from npbrain.utils.measure import *
 
-# profile.py
-from npbrain.utils.profile import *
-
 # run.py
 from npbrain.utils.run import *
 
-# vis.py
-from npbrain.utils.vis import *
+# visualize.py
+from npbrain.utils.visualize import *
 
 
 
